#!/usr/bin/python

import socket, struct, re, string
from clusterpkg.util_ifconfig_module import ifConfig
from clusterpkg.cluster_functions import hostInDNS

TRUE = (1==1)
FALSE = not TRUE

ENFORCE_IP_IN_SAME_NET_AS_HOST = TRUE ### TRUE normally
                                      ### FALSE for testing
class ipConvert:
    "Class for converting dotted-quad IP addresses to long integer and back, and getting network and host portions from an IP address."

    def __init__(self):
        pass

    def dottedQuadToNum(self,ip):
        "convert decimal dotted quad string to long integer"
        ### socket.inet_aton gets an exception on '255.255.255.255', so
        ### trap it here first, and return the long integer that would
        ### have resulted from unpack if inet_aton worked properly.
        if ip == "255.255.255.255":
            return 4294967295L

        #A word of explanation:
        #This conditional is put here to handle the base64 arch case
        #where inet_aton returns an 8 byte value.
        result = socket.inet_aton(ip)
        if(len(result) == 8):
          return struct.unpack('>Q', socket.inet_aton(ip))[0]
        else:
          return struct.unpack('>L', socket.inet_aton(ip))[0]

    def numToDottedQuad(self,n):
        "convert long int to dotted quad string"
        return socket.inet_ntoa(struct.pack('>L', n))

    def makeMask(self,n):
        "return a mask of n bits as a long integer"
        return (2L<<n-1)-1

    def ipToNetAndHost(self, ip, maskbits):
        "return tuple (network, host) dotted-quad addresses given IP and mask size"
        n = self.dottedQuadToNum(ip)
        m = self.makeMask(maskbits)

        #print "\nipToNetAndHost:"
        #print "  ipToNetAndHost in ip       =", ip
        #print "  ipToNetAndHost in maskbits =", maskbits
        #print "  ipToNetAndHost n(dottedQuadToNum(ip)) =", n
        #print "  ipToNetAndHost m(makeMask(maskbits))  =", m

        host = n & m
        net = n - host

        #print
        #print "  ipToNetAndHost host(n & m)   =", host
        #print "  ipToNetAndHost net(n - host) =", net

        dottedQuadNet = self.numToDottedQuad(net)
        dottedQuadHost = self.numToDottedQuad(host)
        #print "  ipToNetAndHost dottedQuad net  =", dottedQuadNet
        #print "  ipToNetAndHost dottedQuad host =", dottedQuadHost
        return dottedQuadNet, dottedQuadHost

    def isHostInNet(self, host_ip, net_ip_with_slash):
        #print "\nisHostInNet:"
        #print "  isHostInNet in: host_ip           =", host_ip
        #print "  isHostInNet in: net_ip_with_slash =", net_ip_with_slash
        net_ip, mask_length = net_ip_with_slash.split('/')
        #print
        #print "  isHostInNet net_ip:     ", net_ip
        #print "  isHostInNet mask_length:", mask_length
        mask_length = int(mask_length)
        net_net, net_host = self.ipToNetAndHost(net_ip, 32-mask_length)
        #print
        #print "  isHostInNet net_net:    ", net_net
        #print "  isHostInNet net_host:   ", net_host
        assert net_host == '0.0.0.0'
        host_net, host_host = self.ipToNetAndHost(host_ip, 32-mask_length)
        #print
        #print "  isHostInNet host_net:   ", host_net
        #print "  isHostInNet host_host:  ", host_host
        #print
        return host_net == net_net

    def splitIpWithSlash(self, ip_with_slash):
        netmask = ip_with_slash
        host = ""
        if string.find(ip_with_slash, "/") != -1:
            ip, mask_length = ip_with_slash.split('/')
            mask_length = int(mask_length)
            netmask, host = self.ipToNetAndHost(ip, 32-mask_length)
        return netmask, host

    def ipIsInSameNetAsHost(self, submitted_ip, submitted_netmask):
        if not ENFORCE_IP_IN_SAME_NET_AS_HOST:
            return TRUE

        ### First, if ip is a DNS name, convert it to dotted quad ###
        if re.search("^\s*[A-Za-z]+", submitted_ip):
            submitted_ip = hostInDNS(submitted_ip)
            if submitted_ip == None:
                return FALSE

        ### Convert ip to number that can be 'and'ed.
        submitted_ip_num = self.dottedQuadToNum(submitted_ip)
        if submitted_netmask in ("", "None", None):
            submitted_netmask_num = ""
        else:
            submitted_netmask, host = self.splitIpWithSlash(submitted_netmask)
            submitted_netmask_num = self.dottedQuadToNum(submitted_netmask)
        #print "TOP: ip          =", submitted_ip
        #print "     ip_num      =", submitted_ip_num
        #print "     netmask     =", submitted_netmask
        #print "     netmask_num =", submitted_netmask_num

        ifconfig = ifConfig()
        interface_list = ifconfig.getAllInterfaceNames()
        interface_ips = {}
        for interface in interface_list:
            iface_ip      = ifconfig.getInetAddr(interface)
            iface_netmask = ifconfig.getNetmask(interface)
            if iface_ip == None or iface_netmask == None:
                continue

            iface_ip_num      = self.dottedQuadToNum(iface_ip)
            iface_netmask_num = self.dottedQuadToNum(iface_netmask)

            iface_ip_netmask_and_result = iface_ip_num & iface_netmask_num

            interface_ips[iface_ip_netmask_and_result] = {
                "ip"          : iface_ip,
                "ip_num"      : iface_ip_num,
                "netmask"     : iface_netmask,
                "netmask_num" : iface_netmask_num
            }

        ### This 'for' just for debugging ###
        for iface_ip_netmask_and_result in interface_ips:
            iface_ip      = interface_ips[iface_ip_netmask_and_result]["ip"]
            iface_ip_num  = interface_ips[iface_ip_netmask_and_result]["ip_num"]
            iface_netmask = interface_ips[iface_ip_netmask_and_result]["netmask"]
            iface_netmask_num = interface_ips[iface_ip_netmask_and_result]["netmask_num"]
            #print "interface ips: ip          = [%s]" % iface_ip
            #print "          ips: ip_num      = [%s]" % iface_ip_num
            #print "          ips: netmask     = [%s]" % iface_netmask
            #print "          ips: netmask_num = [%s]" % iface_netmask_num
            #print "          ips: and result  = [%s]" % iface_ip_netmask_and_result

        if not submitted_netmask in ("", "None", None):
            ### Netmask *WAS* submitted.  Now 'and' submitted ip with
            ### submitted netmask, and see if that result matches the
            ### result of 'and'ing each interface ip with that ip's
            ### netmask.  If *NO* match, then ERROR.
            #print "Netmask *WAS* submitted!!!!!!!!!!"
            if string.find(submitted_netmask, "/") != -1:
                net_ip, mask_length = submitted_netmask.split('/')
                mask_length = int(mask_length)
                net_net, net_host = self.ipToNetAndHost(net_ip, 32-mask_length)
                submitted_netmask = net_net

            submitted_netmask_num = self.dottedQuadToNum(submitted_netmask)
            submitted_ip_netmask_and_result = submitted_ip_num & submitted_netmask_num
            #print "  submitted 'and' = [%s]" % submitted_ip_netmask_and_result
            if interface_ips.has_key(submitted_ip_netmask_and_result):
                #print "Netmask was submitted, and match *FOUND* - OK"
                return TRUE
            else:
                #print "Netmask was submitted, and match *NOT* found - ERROR"
                return FALSE
        else:
            ### Netmask was *NOT* submitted.  'and' submitted ip with
            ### each interface's netmask, and see if result matches the
            ### result of 'and'ing each interface ip with that ip's
            ### netmask.  If *NO* match, then ERROR.
            #print "Netmask was *NOT* submitted ------------>"
            for iface_ip_netmask_and_result in interface_ips:
                iface_netmask_num = interface_ips[iface_ip_netmask_and_result]["netmask_num"]
                ip_netmask_and_result = submitted_ip_num & iface_netmask_num
                #print "  submitted ip num=[%s], iface netmask num=[%s], and=[%s}" % (submitted_ip_num, iface_netmask_num, ip_netmask_and_result)
                if interface_ips.has_key(ip_netmask_and_result):
                    #print "     %%% Netmask was *NOT* submitted, and match *FOUND* - OK"
                    return TRUE

            #print "Netmask was *NOT* submitted, and match *NOT* found - ERROR"
            return FALSE
