#!/usr/bin/python

import os, string, re, copy
from socket import gethostbyaddr

#################################################################
# NOTE: 9/16/2003 HM - commented out 'netbios name' code, since
# a new Samba package was built with a patch to correct the
# 'nmbd -n' option from being ignored.  That new Samba
# package fixes bugzilla 104222.  But I left functions
#    __getHostName
#    __getNetbiosName
# in place - they just aren't used now.
#
# NOTE: 9/12/2003 HM - added code to workaround Samba
# bugzilla 104222.  The -n option in the 'nmbd' command
# is being ignored in Samba, so we added a 'netbios name'
# [global] option.  Added functions:
#    __getHostName
#    __getNetbiosName
#
# When bug 104222 is fixed in Samba, we should be able to
# remove this code.
#################################################################

TRUE = 1
FALSE = 0

SMB_CONF_PATH = "/etc/samba/"
SMB_CONF_FILENAME_PREFIX = "smb.conf."
LOCKDIR = "/var/cache/samba/"
PIDDIR = "/var/run/samba/"

COMMENT = re.compile('^\s*#')
MODULE  = re.compile('^\s*\[')
MODULE_MATCH_PATTERN = re.compile('^\s*\[(.*)\]\s*$')
EQUAL_SIGN = re.compile('=')

### 'netbios name' was an element of GLOBAL_OPTIONS
#                  "netbios name" ]
GLOBAL_OPTIONS = [ "workgroup",
                   "pid directory",
                   "lock directory",
                   "log file",
                   "encrypt passwords",
                   "bind interfaces only",
                   "interfaces" ]

SHARE_OPTIONS = [ "comment",
                  "browsable",
                  "writable",
                  "public",
                  "path" ]

FILE_HEADER = _("""
# Template samba service configuration file - please modify
# to specify subdirectories and client access permissions.
# Remember to copy this file over to *ALL* other cluster
# members.
#
# From a cluster perspective, the key fields are:
# lock directory - must be unique per samba service.
# bind interfaces only - must be present set to yes.
# interfaces - must be set to service floating IP address.
# path - must be the service mountpoint or subdirectory thereof.
# Refer to the cluster documentation for details.
""")

def getAbsShareFilename(sharename):
    return SMB_CONF_PATH + SMB_CONF_FILENAME_PREFIX + sharename

class smbConf:
    "Each cluster service with a Samba share needs its own smb.conf.sharename file.  This class can create or deletes a smb.conf.sharename file."

    def __init__(self, device, device_node=None):
        self.device      = device
        self.device_node = device_node

        sharename = self.device_node.getAttribute("sharename")
        self.sharename_before = sharename # used in writeSambaShareFile to
                                          # determine new, change, or delete
        abs_filename  = self.__getAbsFilename()
        self.share_options = {}
        self.file_exists   = FALSE
        if sharename != "" and os.path.exists(abs_filename):
            ### smb.conf.[sharename] file exists - load it ###
            self.file_exists = TRUE
            file = open(abs_filename, 'r')
            current_section = ""
            for line in file.readlines():
                line = string.strip(line)
                if COMMENT.search(line) or line == "":
                    continue # skip comments and blank lines
                if MODULE.search(line):
                    section_match = MODULE_MATCH_PATTERN.search(line)
                    current_section = section_match.group(1)
                    if current_section == sharename:
                        current_section = "local"
                    self.share_options[current_section] = {}
                elif EQUAL_SIGN.search(line):
                    ### must be a name = value line ###
                    (name, value) = re.split(r'\s*=\s*', line, maxsplit=1)
                    self.share_options[current_section][name] = value
	else:
            ### smb.conf.[sharename] file does *NOT* exist - fill
            ### dictionary with default options
            self.init_file_dict()

    def init_file_dict(self):
        #print "util_smbconf_module init_file_dict TOP!"
        del self.share_options
        options = {}
        options["global"] = {}
        options["global"]["workgroup"]            = "RHCLUSTER"
        options["global"]["lock directory"]       = ""
        options["global"]["log file"]             = "/var/log/samba/%m.log"
        options["global"]["encrypt passwords"]    = "yes"
        options["global"]["bind interfaces only"] = "yes"
        options["global"]["interfaces"]           = ""
        #options["global"]["netbios name"]         = ""

        options["local"] = {}
        options["local"]["comment"]   = "High Availability Samba Service"
        options["local"]["browsable"] = "yes"
        options["local"]["writable"]  = "no"
        options["local"]["public"]    = "yes"
        options["local"]["path"]      = ""

        self.share_options = options
        return

    def __getAbsFilename(self):
        sharename = self.device_node.getAttribute("sharename")
	return getAbsShareFilename(sharename)

    def __getServiceIPAddresses(self):
        service = self.device.parent
        service_ipaddresses = service.get_service_ipaddresses()
        ips = ""
        for service_ipaddress in service_ipaddresses:
            ip = service_ipaddress.get("ipaddress")
            netmask = service_ipaddress.get("netmask")
            if netmask == "" or netmask == "None":
                ip_and_netmask = ip
            else:
                ip_and_netmask = ip + "/" + netmask
            if len(ips) > 0:
                ips = ips + " " # space separate the ip's
            ips = ips + ip_and_netmask
        return ips

    def __getHostName(self, ip):
        try:
            (hostname, aliaslist, ipaddrlist) = gethostbyaddr(ip)
        except:
            return None # host *NOT* found
        return hostname # host found

    def __getNetbiosName(self, ips):
        # ips may contain a space separated list of ipaddresses, so
        #
        # 1. get 1st ipaddress 
        # 2. if ipaddress has a "/netmask", then get just the ipaddress
        # 3. resolve the ipaddress (ex: myhost.redhat.com)
        #      if it resolves, netbios_name = 1st element of hostname ('myhost')
        #      else netbios_name = ipaddress

        # 1. get 1st ipaddress (/netmask)
        tokens = re.split(r'\s*', ips, maxsplit=1)
        ip_and_netmask = tokens[0]

        # 2. if ipaddress has a "/netmask", then get just the ipaddress
        tokens = string.split(ip_and_netmask, "/")
        ip_only = tokens[0]

        # 3. resolve the ipaddress (ex: myhost.redhat.com)
        #      if it resolves, netbios_name = 1st element of hostname ('myhost')
        #      else netbios_name = ipaddress
        hostname = self.__getHostName(ip_only)
        netbios_name = ip_only
        if hostname != None:
            tokens = string.split(hostname, '.')
            netbios_name = tokens[0]
        return netbios_name

    def __getDeviceMountpoint(self):
        mount = self.device.get_mount()
        mountpoint = mount.get("mountpoint")
        return mountpoint

    def writeSambaShareFile(self):
        "Write an existing smb.conf.[sharename] file."
        ipaddresses = self.__getServiceIPAddresses()
        mountpoint  = self.__getDeviceMountpoint()

        sharename = self.device_node.getAttribute("sharename")
        if sharename in ("", "None", "none"):
            sharename   = ""
            ipaddresses = ""
            mountpoint  = ""
            action      = "none"
            return (sharename, ipaddresses, mountpoint, action)

        #netbios_name = self.__getNetbiosName(ipaddresses)

        self.share_options["global"]["interfaces"] = ipaddresses
        self.share_options["global"]["pid directory"] = PIDDIR + \
                                                         sharename
        self.share_options["global"]["lock directory"] = LOCKDIR + \
                                                         sharename
        #self.share_options["global"]["netbios name"] = netbios_name
        self.share_options["local"]["path"] = mountpoint

        ### Write out file ###
        options_left = copy.deepcopy(self.share_options)
        abs_filename  = self.__getAbsFilename()
        file = open(abs_filename, 'w')
        file.write(FILE_HEADER)

        global_options_left = copy.deepcopy(self.share_options["global"])
        file.write("\n[global]\n")
        ### print [global] options in order ###
        for option in GLOBAL_OPTIONS:
            value = self.share_options["global"][option]
            file.write("\t%s = %s\n" % (option, value))
            del global_options_left[option]
        for option, value in global_options_left.items():
            file.write("\t%s = %s\n" % (option, value))
        del options_left["global"]

        file.write("\n")

        share_options_left = copy.deepcopy(self.share_options["local"])
        file.write("[%s]\n" % sharename)
        ### print [sharename] options in order ###
        for option in SHARE_OPTIONS:
            value = self.share_options["local"][option]
            file.write("\t%s = %s\n" % (option, value))
            del share_options_left[option]
        for option, value in share_options_left.items():
            file.write("\t%s = %s\n" % (option, value))
        del options_left["local"]

        if len(options_left) > 0:
            for section, section_options in options_left.items():
                file.write("\n")
                file.write("[%s]\n" % section)
                for option, value in section_options.items():
                    file.write("\t%s = %s\n" % (option, value))

        if self.sharename_before in ("", "None", "none"):
            self.sharename_before = ""
        if sharename in ("", "None", "none"):
            sharename = ""
        action = "none"
        if self.sharename_before != sharename:
            if self.sharename_before == "":
                action = "new"
            elif sharename == "":
                action = "delete"
            else:
                action = "change"
        return (sharename, ipaddresses, mountpoint, action)

    def deleteSambaShareFile(self):
        try:
            os.remove(self.__getAbsFilename())
        except OSError, e:
            pass
        ### Invoke __init__ now to reinitialize object data, since
        ### share is gone, but device still exists.
        self.__init__()
